import Flutter
import UIKit
import FawryFrameworkAnonymous


public class SwiftFawrySdkPlugin: NSObject, FlutterPlugin {
    
    private let METHOD_START_PAYMENT : String = "init"
    private let LAUNCH_MODEL_JSON_ARG = "launchModelJson"
    private let BASE_URL_ARG = "baseURL"
    private let LANGUAGE_ARG = "lang"
    private let DART_LANGUAGE_ARABIC = "ARABIC";
    private let DART_LANGUAGE_ENGLISH = "ENGLISH";
    private static let CHANNEL_NAME = "channel/fawry_sdk"
    private static var EVENT_CHANNEL_CALLBACK_RESULT : String = "event.channel/fawry_sdk" ;
    
    private static var eventSink : FlutterEventSink!
    
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: CHANNEL_NAME, binaryMessenger: registrar.messenger())
        let instance = SwiftFawrySdkPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
        let eventChannel = FlutterEventChannel(name: EVENT_CHANNEL_CALLBACK_RESULT, binaryMessenger: registrar.messenger())
        eventChannel.setStreamHandler(FawryStreamHandler())
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        
        if call.method == METHOD_START_PAYMENT {
            let args = call.arguments as! [String:Any?]
            guard let fawryModelString = args["launchModelJson"] as? String else {
                // TODO should return error
                let responseStatus = ResponseStatus(status: 1, message: "faild to map the model")
                SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString)
                
                return
            }
            guard let baseURL = args[BASE_URL_ARG] as? String else {
                return
            }
            guard let dartLang = args[LANGUAGE_ARG] as? String else {
                return
            }
            
            let decoder = JSONDecoder()
            guard let jsonData = fawryModelString.data(using: .utf8) else {
                let responseStatus = ResponseStatus(status: 1, message: "faild to map the model")
                SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString)
                return
            }
            do {
                let fawryLaunchModel = try decoder.decode(FawryLaunchModel.self, from: jsonData)
                let lang = (dartLang == DART_LANGUAGE_ARABIC ? (AppLanguage.Arabic) : (AppLanguage.English))
                showFawrySDK(launchModel: fawryLaunchModel,baseURL: baseURL, lang: lang)
            } catch {
                // TODO return ERROR
                let responseStatus = ResponseStatus(status: 1, message: error.localizedDescription)
                SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString)            }
        }
    }
    
    func showFawrySDK (launchModel: FawryLaunchModel, baseURL: String, lang: String) {
        guard let fawryInstance = FrameworkHelper.shared else { return }
        guard let viewController = UIApplication.shared.delegate?.window??.rootViewController else { return }
        fawryInstance.launchAnonymousSDK(on: viewController,
                                         launchModel: launchModel ,
                                         baseURL: baseURL, appLanguage: lang,
                                         enable3Ds: true,
                                         completionBlock: { (status) in
            guard let status = status else {return}
            let responseStatus = ResponseStatus(status: status.rawValue, message: "Payment Method: SDK Loaded")
            SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString)
        }, onPreCompletionHandler: { (error) in
            let responseStatus = ResponseStatus(status: error?.errorCode ?? 1, message: error?.description ?? "")
            SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString)
        }, errorBlock: { (error) in
            let responseStatus = ResponseStatus(status: error?.errorCode ?? 1, message: error?.description ?? "")
            SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString)
        }, onSuccessHandler: { (response) in
            guard let responseCharge = response as? PaymentChargeResponse else {
                return}
            SwiftFawrySdkPlugin.eventSink(responseCharge.convertToString)
        })
    }
    
    class FawryStreamHandler: NSObject, FlutterStreamHandler {
        public func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
            eventSink = events
            return nil
        }
        
        public func onCancel(withArguments arguments: Any?) -> FlutterError? {
            eventSink = nil
            return nil
        }
    }
    
}
